#!/usr/bin/env python

# Color correction by setting levels' black/white/gray points like described in tutorial
# found here: http://blog.patdavid.net/2011/06/getting-around-in-gimp-color-correction.html
# Author: Tin Tran 
# Date: 2014
# This code has been translated from script-fu i wrote that was too slow
# so moving it to python to see if it's any faster
import math
import string
#import Image
from gimpfu import *
from array import array

def gray_pt_to_gamma(gray_color, gray_pt, black_pt, white_pt) :
	luminosity = (0.2126 * gray_color[0]) + ( 0.7152 * gray_color[1]) + (0.0722 * gray_color[2])
	range = white_pt - black_pt
	offset = gray_pt - black_pt
	return_value = math.log(1.0*offset/range)/math.log(1.0*luminosity/range)
	return (return_value)

def absolute_difference(pixelcolor, definedcolor) :
	return_value = abs(pixelcolor[0] - definedcolor[0]) + abs(pixelcolor[1] - definedcolor[1]) + abs(pixelcolor[2] - definedcolor[2])
	return (return_value)
	
def get_pixel(x,y,image_width,p_size,src_pixels) :
	src_pos = (x + image_width * y) * p_size
	return (src_pixels[src_pos: src_pos + p_size])
	
def python_color_correction(image, layer) :
	
	pdb.gimp_image_undo_group_start(image)
	current_white_difference = 765
	current_black_difference = 765
	current_gray_difference = 765
	current_white_location = [0,0]
	current_black_location = [0,0]
	current_gray_location = [0,0]
	white = [255,255,255]
	black = [0,0,0]
	gray = [128,128,128]
	image_width = pdb.gimp_image_width(image)
	image_height = pdb.gimp_image_height(image)
	pdb.gimp_message("Width:" + str(layer.width))
	pdb.gimp_message("Height:" + str(layer.height))
	srcRgn = layer.get_pixel_rgn(0, 0, image_width, image_height,False, False)
	src_pixels = array("B", srcRgn[0:image_width, 0:image_height])
	p_size = len(srcRgn[0,0])
	x = 0
	y = 0
	gimp.progress_init("Color correction detecting black/white/gray...")
	for y in xrange(0, (image_height-1)) :
		for x in xrange(0, (image_width-1)) :
			gimp.progress_update(1.0 * y/image_height)
			src_pos = (x + image_width * y) * p_size
			current_pixel = src_pixels[src_pos: src_pos + p_size]
			if (absolute_difference(current_pixel,white) < current_white_difference) :
				current_white_difference = absolute_difference(current_pixel,white)
				current_white_location[0] = x
				current_white_location[1] = y
			if (absolute_difference(current_pixel,black) < current_black_difference) :
				current_black_difference = absolute_difference(current_pixel,black)
				current_black_location[0] = x
				current_black_location[1] = y
			if (absolute_difference(current_pixel,gray) < current_gray_difference) :
				current_gray_difference = absolute_difference(current_pixel,gray)
				current_gray_location[0] = x
				current_gray_location[1] = y
	white_pix = get_pixel(current_white_location[0],current_white_location[1],image_width,p_size,src_pixels)
	black_pix = get_pixel(current_black_location[0],current_black_location[1],image_width,p_size,src_pixels)
	gray_pix = get_pixel(current_gray_location[0],current_gray_location[1],image_width,p_size,src_pixels)
	#Set RED levels
	pdb.gimp_levels(layer,1,black_pix[0],white_pix[0],gray_pt_to_gamma(gray_pix,gray_pix[0],black_pix[0],white_pix[0]),0,255)
	#Set GREEN levels
	pdb.gimp_levels(layer,2,black_pix[1],white_pix[1],gray_pt_to_gamma(gray_pix,gray_pix[1],black_pix[1],white_pix[1]),0,255)
	#Set BLUE levels
	pdb.gimp_levels(layer,3,black_pix[2],white_pix[2],gray_pt_to_gamma(gray_pix,gray_pix[2],black_pix[2],white_pix[2]),0,255)
	
	pdb.gimp_message("Color Correction Complete!")
	pdb.gimp_progress_end()
	pdb.gimp_image_undo_group_end(image)
	
	pdb.gimp_displays_flush()
    #return

register(
    "python_fu_color_correction",
    "Auto Color Correction",
    "Auto Color Correction by setting levels' black/white/gray points",
    "Tin Tran",
    "Tin Tran",
    "2014",
    "<Image>/Colors/Auto/Color Correction",
    "RGB*, GRAY*",      # Create a new image, don't work on an existing one
    [
	#(PF_FILE, "infilename", "Temp Filepath", "/Default/Path")
	#(PF_DIRNAME, "source_directory", "Source Directory", "") for some reason, on my computer when i(Tin) use PF_DIRNAME the pythonw.exe would crash
	],
    [],
    python_color_correction)

main()
